export interface IDatabase {
    insert(tableName: string, values: {
        [key: string]: any;
    }): number;
    replace(tableName: string, values: {
        [key: string]: any;
    }): number;
    update(tableName: string, values: {
        [key: string]: any;
    }, whereClause: string, whereArs: Array<any>): number;
    delete(tableName: string, whereClause?: string, whereArs?: Array<any>): number;
    select(sql: string, conditionParams?: Array<any>): ExtendedPromise;
    query(tableName: string, columns?: Array<string>, selection?: string, selectionArgs?: Array<any>, groupBy?: string, orderBy?: string, limit?: string): ExtendedPromise;
    execSQL(sql: string): void;
    beginTransact(): void;
    commit(): void;
    rollback(): void;
    close(): void;
}
export declare const enum ReturnType {
    AS_OBJECT = 0,
    AS_ARRAY = 1
}
export interface DbCreationOptions {
    version?: number;
    createTableScriptsFn?: () => Array<string>;
    dropTableScriptsFn?: () => Array<string>;
    returnType?: ReturnType;
}
export declare function parseToDbValue(value: any): any;
export declare function parseToJsValue(value: any): any;
export declare class ExtendedPromise {
    private _subscribers;
    private _executorFn;
    constructor(executorFn: ExecutorType);
    map(callback: mapCallbackType): Promise<any>;
    reduce(callback: reduceCallbackType, initialValue: any): Promise<any>;
    process(): Promise<any>;
}
declare type ExecutorType = (subscribers: Array<mapCallbackType | reduceSubscribersType>, resolve: Function, reject: Function) => void;
declare type mapCallbackType = (row: any, index: number) => any;
declare type reduceCallbackType = (accumulator: any, row: any, index: number) => any;
declare type reduceSubscribersType = {
    callback: reduceCallbackType;
    initialValue?: any;
};
export {};
